@@ -20,27 +20,35 @@ __all__ = ['bitarray', 'frozenbitarray', 'decodetree', '__version__']
 
 
 class frozenbitarray(bitarray):
-    """frozenbitarray(initializer=0, /, endian='big') -> frozenbitarray
+    """frozenbitarray(initializer=0, /, endian='big', buffer=None) -> \
+frozenbitarray
 
 Return a frozenbitarray object, which is initialized the same way a bitarray
 object is initialized.  A frozenbitarray is immutable and hashable.
 Its contents cannot be altered after it is created; however, it can be used
 as a dictionary key.
 """
+    def __init__(self, *args, **kwargs):
+        self._freeze()
+
     def __repr__(self):
         return 'frozen' + bitarray.__repr__(self)
 
     def __hash__(self):
         "Return hash(self)."
         if getattr(self, '_hash', None) is None:
-            # ensure hash is independent of endianness
-            a = bitarray(self, 'big') if self.endian() == 'little' else self
+            # ensure hash is independent of endianness, also the copy will be
+            # mutable such that .tobytes() can zero out the pad bits
+            a = bitarray(self, 'big')
             self._hash = hash((len(a), a.tobytes()))
         return self._hash
 
+    # Technically the code below is not necessary, as all these methods will
+    # raise a TypeError on read-only memory.  However, with a different error
+    # message.
     def __delitem__(self, *args, **kwargs):
         ""  # no docstring
-        raise TypeError("'frozenbitarray' is immutable")
+        raise TypeError("frozenbitarray is immutable")
 
     append = bytereverse = clear = extend = encode = fill = __delitem__
     frombytes = fromfile = insert = invert = pack = pop = __delitem__
