@@ -57,16 +57,18 @@ ensure_ba_of_length(PyObject *a, const Py_ssize_t n)
 static Py_ssize_t
 count_to_n(bitarrayobject *a, Py_ssize_t n)
 {
+    const Py_ssize_t nbits = a->nbits;
     Py_ssize_t i = 0;        /* index */
     Py_ssize_t j = 0;        /* total count up to index */
     Py_ssize_t block_start, block_stop, k, m;
 
+    assert(0 <= n && n <= nbits);
     if (n == 0)
         return 0;
 
 #define BLOCK_BITS  8192
     /* by counting big blocks we save comparisons */
-    while (i + BLOCK_BITS < a->nbits) {
+    while (i + BLOCK_BITS < nbits) {
         m = 0;
         assert(i % 8 == 0);
         block_start = i >> 3;
@@ -81,7 +83,7 @@ count_to_n(bitarrayobject *a, Py_ssize_t n)
     }
 #undef BLOCK_BITS
 
-    while (i + 8 < a->nbits) {
+    while (i + 8 < nbits) {
         k = i >> 3;
         assert(k < Py_SIZE(a));
         m = bitcount_lookup[(unsigned char) a->ob_item[k]];
@@ -91,7 +93,7 @@ count_to_n(bitarrayobject *a, Py_ssize_t n)
         i += 8;
     }
 
-    while (j < n && i < a->nbits ) {
+    while (j < n && i < nbits ) {
         j += getbit(a, i);
         i++;
     }
@@ -148,7 +150,7 @@ find_last(bitarrayobject *self, int vi, Py_ssize_t a, Py_ssize_t b)
     assert(0 <= a && a <= self->nbits);
     assert(0 <= b && b <= self->nbits);
     assert(0 <= vi && vi <= 1);
-    if (self->nbits == 0 || n <= 0)
+    if (n <= 0)
         return -1;
 
     /* the logic here is the same as in find_bit() in _bitarray.c */
@@ -177,7 +179,7 @@ find_last(bitarrayobject *self, int vi, Py_ssize_t a, Py_ssize_t b)
             return res;
 
         for (i = byte_b - 1; i >= byte_a; i--) {  /* skip bytes */
-            assert(0 <= i && i < Py_SIZE(self));
+            assert_byte_in_range(self, i);
             if (c ^ self->ob_item[i])
                 return find_last(self, vi, BITS(i), BITS(i) + 8);
         }
@@ -194,31 +196,32 @@ find_last(bitarrayobject *self, int vi, Py_ssize_t a, Py_ssize_t b)
 static PyObject *
 r_index(PyObject *module, PyObject *args)
 {
-    PyObject *v = Py_True, *a;
-    Py_ssize_t i;
+    PyObject *value = Py_True, *a;
+    Py_ssize_t start = 0, stop = PY_SSIZE_T_MAX, res;
     int vi;
 
-    if (!PyArg_ParseTuple(args, "O|O:rindex", &a, &v))
+    if (!PyArg_ParseTuple(args, "O|Onn:rindex", &a, &value, &start, &stop))
         return NULL;
     if (ensure_bitarray(a) < 0)
         return NULL;
-
-    if ((vi = pybit_as_int(v)) < 0)
+    if ((vi = pybit_as_int(value)) < 0)
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
-    i = find_last(aa, vi, 0, aa->nbits);
+    normalize_index(aa->nbits, &start);
+    normalize_index(aa->nbits, &stop);
+    res = find_last(aa, vi, start, stop);
 #undef aa
-    if (i < 0)
+    if (res < 0)
         return PyErr_Format(PyExc_ValueError, "%d not in bitarray", vi);
 
-    return PyLong_FromSsize_t(i);
+    return PyLong_FromSsize_t(res);
 }
 
 PyDoc_STRVAR(rindex_doc,
-"rindex(bitarray, value=1, /) -> int\n\
+"rindex(bitarray, value=1, start=0, stop=<end of array>, /) -> int\n\
 \n\
-Return the rightmost index of `value` in bitarray.\n\
+Return the rightmost (highest) index of `value` in bitarray.\n\
 Raises `ValueError` if the value is not present.");
 
 /* --------------------------- unary functions ------------------------- */
@@ -226,17 +229,17 @@ Raises `ValueError` if the value is not present.");
 static PyObject *
 parity(PyObject *module, PyObject *a)
 {
-    Py_ssize_t i, nbytes;
     unsigned char par = 0;
+    Py_ssize_t i;
 
     if (ensure_bitarray(a) < 0)
         return NULL;
 
-    nbytes = Py_SIZE(a);
 #define aa  ((bitarrayobject *) a)
-    setunused(aa);
-    for (i = 0; i < nbytes; i++)
+    for (i = 0; i < aa->nbits / 8; i++)
         par ^= aa->ob_item[i];
+    if (aa->nbits % 8)
+        par ^= zeroed_last_byte(aa);
 #undef aa
 
     return PyLong_FromLong((long) bitcount_lookup[par] % 2);
@@ -260,9 +263,10 @@ enum kernel_type {
 static PyObject *
 binary_function(PyObject *args, enum kernel_type kern, const char *format)
 {
-    Py_ssize_t res = 0, nbytes, i;
+    Py_ssize_t res = 0, s, i;
     PyObject *a, *b;
     unsigned char c;
+    int r;
 
     if (!PyArg_ParseTuple(args, format, &a, &b))
         return NULL;
@@ -281,36 +285,55 @@ binary_function(PyObject *args, enum kernel_type kern, const char *format)
                         "bitarrays of equal endianness expected");
         return NULL;
     }
-    setunused(aa);
-    setunused(bb);
-    assert(Py_SIZE(a) == Py_SIZE(b));
-    nbytes = Py_SIZE(a);
+    s = aa->nbits / 8;       /* number of whole bytes in buffer */
+    r = aa->nbits % 8;       /* remaining bits  */
 
     switch (kern) {
     case KERN_cand:
-        for (i = 0; i < nbytes; i++) {
+        for (i = 0; i < s; i++) {
             c = aa->ob_item[i] & bb->ob_item[i];
             res += bitcount_lookup[c];
         }
+        if (r) {
+            c = zeroed_last_byte(aa) & zeroed_last_byte(bb);
+            res += bitcount_lookup[c];
+        }
         break;
+
     case KERN_cor:
-        for (i = 0; i < nbytes; i++) {
+        for (i = 0; i < s; i++) {
             c = aa->ob_item[i] | bb->ob_item[i];
             res += bitcount_lookup[c];
         }
-        break;
-    case KERN_cxor:
-        for (i = 0; i < nbytes; i++) {
-            c = aa->ob_item[i] ^ bb->ob_item[i];
+        if (r) {
+            c = zeroed_last_byte(aa) | zeroed_last_byte(bb);
             res += bitcount_lookup[c];
         }
         break;
+
+    case KERN_cxor:
+        for (i = 0; i < s; i++) {
+            c = aa->ob_item[i] ^ bb->ob_item[i];
+            res += bitcount_lookup[c];
+        }
+        if (r) {
+            c = zeroed_last_byte(aa) ^ zeroed_last_byte(bb);
+            res += bitcount_lookup[c];
+        }
+        break;
+
     case KERN_subset:
-        for (i = 0; i < nbytes; i++) {
+        for (i = 0; i < s; i++) {
             if ((aa->ob_item[i] & bb->ob_item[i]) != aa->ob_item[i])
                 Py_RETURN_FALSE;
         }
+        if (r) {
+            if ((zeroed_last_byte(aa) & zeroed_last_byte(bb)) !=
+                 zeroed_last_byte(aa))
+                Py_RETURN_FALSE;
+        }
         Py_RETURN_TRUE;
+
     default:  /* cannot happen */
         return NULL;
     }
@@ -357,22 +380,20 @@ serialize(PyObject *module, PyObject *a)
 {
     PyObject *result;
     Py_ssize_t nbytes;
-    char *data;
+    char *str;
 
     if (ensure_bitarray(a) < 0)
         return NULL;
 
     nbytes = Py_SIZE(a);
-    data = (char *) PyMem_Malloc((size_t) nbytes + 1);
-    if (data == NULL)
+    if ((result = PyBytes_FromStringAndSize(NULL, nbytes + 1)) == NULL)
         return PyErr_NoMemory();
 
+    str = PyBytes_AsString(result);
 #define aa  ((bitarrayobject *) a)
-    *data = (char) (16 * IS_BE(aa) + setunused(aa));
-    memcpy(data + 1, aa->ob_item, (size_t) nbytes);
+    *str = (char) (16 * IS_BE(aa) + setunused(aa));
+    memcpy(str + 1, aa->ob_item, (size_t) nbytes);
 #undef aa
-    result = PyBytes_FromStringAndSize(data, nbytes + 1);
-    PyMem_Free((void *) data);
     return result;
 }
 
@@ -426,6 +447,7 @@ ba2hex(PyObject *module, PyObject *a)
     be = IS_BE(aa);
     for (i = 0; i < strsize; i += 2) {
         unsigned char c = aa->ob_item[i / 2];
+
         str[i + le] = hexdigits[c >> 4];
         str[i + be] = hexdigits[0x0f & c];
     }
@@ -451,7 +473,7 @@ hex2ba(PyObject *module, PyObject *args)
     PyObject *a;
     char *str;
     Py_ssize_t i, strsize;
-    int le, be, x, y;
+    int le, be;
 
     if (!PyArg_ParseTuple(args, "Os#", &a, &str, &strsize))
         return NULL;
@@ -463,8 +485,9 @@ hex2ba(PyObject *module, PyObject *args)
     be = IS_BE(aa);
     assert(le + be == 1 && str[strsize] == 0);
     for (i = 0; i < strsize; i += 2) {
-        x = hex_to_int(str[i + le]);
-        y = hex_to_int(str[i + be]);
+        int x = hex_to_int(str[i + le]);
+        int y = hex_to_int(str[i + be]);
+
         if (x < 0 || y < 0) {
             /* ignore the terminating NUL - happends when strsize is odd */
             if (i + le == strsize) /* str[i+le] is NUL */
@@ -499,6 +522,7 @@ digit_to_int(char c, int n)
 {
     if (n <= 16) {              /* base 2, 4, 8, 16 */
         int i = hex_to_int(c);
+
         if (0 <= i && i < n)
             return i;
     }
@@ -544,7 +568,7 @@ ba2base(PyObject *module, PyObject *args)
     PyObject *result, *a;
     size_t i, strsize;
     char *str;
-    int n, m, x, k, le;
+    int n, m, le;
 
     if (!PyArg_ParseTuple(args, "iO:ba2base", &n, &a))
         return NULL;
@@ -570,9 +594,12 @@ ba2base(PyObject *module, PyObject *args)
 
     le = IS_LE(aa);
     for (i = 0; i < strsize; i++) {
-        x = 0;
-        for (k = 0; k < m; k++)
-            x |= getbit(aa, m * i + (le ? k : (m - k - 1))) << k;
+        int j, k, x = 0;
+
+        for (j = 0; j < m; j++) {
+            k = le ? j : (m - j - 1);
+            x |= getbit(aa, i * m + k) << j;
+        }
         str[i] = alphabet[x];
     }
     result = Py_BuildValue("s#", str, strsize);
@@ -605,7 +632,7 @@ base2ba(PyObject *module, PyObject *args)
     PyObject *a;
     Py_ssize_t i, strsize;
     char *str;
-    int n, m, d, k, le;
+    int n, m, le;
 
     if (!PyArg_ParseTuple(args, "iOs#", &n, &a, &str, &strsize))
         return NULL;
@@ -619,13 +646,16 @@ base2ba(PyObject *module, PyObject *args)
 
     le = IS_LE(aa);
     for (i = 0; i < strsize; i++) {
-        d = digit_to_int(str[i], n);
+        int j, k, d = digit_to_int(str[i], n);
+
         if (d < 0) {
             PyErr_SetString(PyExc_ValueError, "invalid digit found");
             return NULL;
         }
-        for (k = 0; k < m; k++)
-            setbit(aa, m * i + (le ? k : (m - k - 1)), d & (1 << k));
+        for (j = 0; j < m; j++) {
+            k = le ? j : (m - j - 1);
+            setbit(aa, i * m + k, d & (1 << j));
+        }
     }
 #undef aa
     Py_RETURN_NONE;
@@ -656,7 +686,7 @@ static PyObject *
 vl_decode(PyObject *module, PyObject *args)
 {
     const Py_ssize_t ibits = 256;    /* initial number of bits in a */
-    PyObject *iter, *item, *res, *a;
+    PyObject *iter, *item, *a;
     Py_ssize_t padding = 0;  /* number of pad bits read from header byte */
     Py_ssize_t i = 0;        /* bit counter */
     unsigned char b = 0x80;  /* empty stream will raise StopIteration */
@@ -693,14 +723,16 @@ vl_decode(PyObject *module, PyObject *args)
 
         assert(i == 0 || (i + PADBITS) % 7 == 0);
         if (i == aa->nbits) {
+            PyObject *ret;  /* return object from .frombytes() call */
+
             /* grow memory - see above */
             assert(i % 8 == 0);  /* added dummy bytes are aligned */
             /* 63 is a multiple of 7 - bytes will be aligned for next call */
-            res = PyObject_CallMethod(a, "frombytes", BYTES_SIZE_FMT,
+            ret = PyObject_CallMethod(a, "frombytes", BYTES_SIZE_FMT,
                                       base64_alphabet, (Py_ssize_t) 63);
-            if (res == NULL)
+            if (ret == NULL)
                 return NULL;
-            Py_DECREF(res);  /* drop extend result */
+            Py_DECREF(ret);  /* drop frombytes result */
         }
         assert(i + 6 < aa->nbits);
 
@@ -740,7 +772,7 @@ static PyObject *
 vl_encode(PyObject *module, PyObject *a)
 {
     PyObject *result;
-    Py_ssize_t padding, n, m, i, k;
+    Py_ssize_t padding, n, m, i;
     Py_ssize_t j = 0;           /* byte conter */
     char *str;
 
@@ -753,16 +785,18 @@ vl_encode(PyObject *module, PyObject *a)
     padding = m - aa->nbits;  /* number of pad bits */
     assert(0 <= padding && padding < 7);
 
-    if ((str = (char *) PyMem_Malloc((size_t) n)) == NULL)
+    if ((result = PyBytes_FromStringAndSize(NULL, n)) == NULL)
         return PyErr_NoMemory();
 
+    str = PyBytes_AsString(result);
     str[0] = aa->nbits > 4 ? 0x80 : 0x00;  /* leading bit */
     str[0] |= padding << 4;                /* encode padding */
     for (i = 0; i < 4 && i < aa->nbits; i++)
         str[0] |= (0x08 >> i) * getbit(aa, i);
 
     for (i = 4; i < aa->nbits; i++) {
-        k = (i - 4) % 7;
+        int k = (i - 4) % 7;
+
         if (k == 0) {
             j++;
             str[j] = j < n - 1 ? 0x80 : 0x00;  /* leading bit */
@@ -772,8 +806,6 @@ vl_encode(PyObject *module, PyObject *a)
 #undef aa
     assert(j == n - 1);
 
-    result = PyBytes_FromStringAndSize(str, n);
-    PyMem_Free((void *) str);
     return result;
 }
 
